@@ -40,9 +40,13 @@ def dbt_manifest(ctx, manifest_file, project_name, batch_size, default_resource,
 @click.argument('RUN_RESULTS_FILE', required=True, type=click.Path(exists=True))
 @click.option('--project-name', required=False, type=click.STRING,
               help='Name that uniquely identifies dbt project.')
+@click.option('--async/--no-async', 'do_async', default=True, show_default=True,
+              help='Toggle asynchronous processing of dbt run results file')
 @click.pass_obj
-def dbt_run_results(ctx, run_results_file, project_name):
-    DbtImporter(mc_client=create_mc_client(ctx), print_func=click.echo).import_run_results(
+def dbt_run_results(ctx, run_results_file, project_name, do_async):
+    importer = DbtImporter(mc_client=create_mc_client(ctx), print_func=click.echo)
+    import_func = importer.upload_run_results if do_async else importer.import_run_results
+    import_func(
         dbt_run_results=run_results_file,
         project_name=project_name
     )
@@ -65,4 +69,3 @@ def dbt_cloud(ctx, project_id, job_id, manifest_only, default_resource):
         manifest_only=manifest_only,
         default_resource=default_resource
     )
-
