@@ -62,14 +62,33 @@ def role_options(required: bool = True) -> List:
     ]
 
 
-# Name is used for the creation of the warehouse
-WAREHOUSE_CREATE_OPTIONS = [
-    click.option('--name', help='Friendly name for the created warehouse. Name must be unique.', required=False)
-]
-# Name is used to identify the warehouse that will contain the connection.
-WAREHOUSE_SELECT_OPTIONS = [
-    click.option('--name', help='Friendly name of the warehouse which the connection will belong to.', required=False)
-]
+# Name is used for the creation of a warehouse that will contain the connection.
+def warehouse_create_option(required: bool = False, default: str = None) -> List:
+    return _warehouse_option(
+        help='Friendly name for the created warehouse. Name must be unique.',
+        required=required,
+        default=default
+    )
+
+
+# Name is used to identify an existing warehouse that will contain the connection.
+def warehouse_select_option(required: bool = False, default: str = None) -> List:
+    return _warehouse_option(
+        help='Friendly name of the warehouse which the connection will belong to.',
+        required=required,
+        default=default
+    )
+
+
+def _warehouse_option(help: str, required: bool = False, default: str = None) -> List:
+    return [click.option(
+        '--name',
+        help=help,
+        required=required,
+        default=default
+    )]
+
+
 WAREHOUSE_OPTIONAL_CREATE_OPTIONS = [
     click.option('--create-warehouse', help='Create a new warehouse with this connection', type=click.BOOL, default=True, required=False)
 ]
@@ -78,12 +97,6 @@ S3_OPTIONS = [
     click.option('--prefix', help='Path to query logs.', required=True),
     *role_options(required=False)
 ]
-EVENT_OPTIONS = [
-    click.option('--enable/--disable', 'toggle', help='Enable or disable events. Enables if not specified.',
-                 default=True),
-    click.option('--connection-id', help=CONNECTION_ID_VERBIAGE, required=False, type=click.UUID,
-                 callback=convert_uuid_callback)
-]
 DATABASE_OPTIONS = [
     click.option('--host', help='Hostname.', required=True),
     click.option('--user', help='Username with access to the database.', required=True),
@@ -195,7 +208,7 @@ def integrations():
               default=False, show_default=True)
 @click.option('--catalog', help='Presto catalog name. For using multiple hive clusters with Presto. '
                                 'Uses \'hive\' if not specified.', required=False)
-@add_common_options(WAREHOUSE_CREATE_OPTIONS)
+@add_common_options(warehouse_create_option())
 @add_common_options(WAREHOUSE_OPTIONAL_CREATE_OPTIONS)
 @add_common_options(DATABASE_OPTIONS)
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@@ -224,7 +237,7 @@ def add_hive_metastore(ctx, database, name, **kwargs):
 @click.option('--cert-s3', help='Object path (key) to a certificate already uploaded to the collector.',
               required=False, cls=AdvancedOptions, mutually_exclusive_options=['cert_file'])
 @click.option('--skip-cert-verification', help='Skip SSL certificate verification.', required=False, is_flag=True)
-@add_common_options(WAREHOUSE_SELECT_OPTIONS)
+@add_common_options(warehouse_select_option())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_presto(ctx, password, name, **kwargs):
@@ -246,7 +259,7 @@ def add_presto(ctx, password, name, **kwargs):
 @click.option('--user', help='Username with access to hive.', required=True)
 @click.option('--auth-mode', help='Hive authentication mode.', required=False, default='SASL',
               type=click.Choice(['SASL', 'NOSASL']), show_default=True)
-@add_common_options(WAREHOUSE_SELECT_OPTIONS)
+@add_common_options(warehouse_select_option())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_hive(ctx, user, name, **kwargs):
@@ -283,7 +296,7 @@ def add_hive_logs(ctx, role, **kwargs):  # DEPRECATED
 @click.pass_obj
 @click.option('--region', help=f'Glue catalog region. {REGION_VERBIAGE}.', required=False)
 @add_common_options(role_options())
-@add_common_options(WAREHOUSE_CREATE_OPTIONS)
+@add_common_options(warehouse_create_option())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_glue(ctx, role, region, name, **kwargs):
@@ -302,7 +315,7 @@ def add_glue(ctx, role, region, name, **kwargs):
               help='Workbook for running queries and retrieving logs. If not specified the primary is used.',
               required=False)
 @click.option('--region', help=f'Athena cluster region. {REGION_VERBIAGE}.', required=False)
-@add_common_options(WAREHOUSE_SELECT_OPTIONS)
+@add_common_options(warehouse_select_option())
 @add_common_options(role_options())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
@@ -323,7 +336,7 @@ def add_athena(ctx, role, region, name, **kwargs):
 @click.option('--user', help='Username with access to spark.', required=True)
 @click.option('--password', help=f'User\'s password. {PASSWORD_VERBIAGE}.', required=True,
               cls=AdvancedOptions, prompt_if_requested=True)
-@add_common_options(WAREHOUSE_SELECT_OPTIONS)
+@add_common_options(warehouse_select_option())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_spark_binary_mode(ctx, user, name, **kwargs):
@@ -341,7 +354,7 @@ def add_spark_binary_mode(ctx, user, name, **kwargs):
 @click.option('--user', help='Username with access to spark.', required=True)
 @click.option('--password', help=f'User\'s password. {PASSWORD_VERBIAGE}.', required=True,
               cls=AdvancedOptions, prompt_if_requested=True)
-@add_common_options(WAREHOUSE_SELECT_OPTIONS)
+@add_common_options(warehouse_select_option())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_spark_http_mode(ctx, user, name, **kwargs):
@@ -356,7 +369,7 @@ def add_spark_http_mode(ctx, user, name, **kwargs):
 @integrations.command(help=f'Setup a Spark integration for Databricks. {SQL_VERBIAGE}.')
 @click.pass_obj
 @add_common_options(DATABRICKS_OPTIONS)
-@add_common_options(WAREHOUSE_SELECT_OPTIONS)
+@add_common_options(warehouse_select_option())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_spark_databricks(ctx, name, **kwargs):
@@ -370,7 +383,7 @@ def add_spark_databricks(ctx, name, **kwargs):
 
 @integrations.command(help='Setup a Databricks metastore integration. For metadata and health queries.')
 @click.pass_obj
-@add_common_options(WAREHOUSE_CREATE_OPTIONS)
+@add_common_options(warehouse_create_option())
 @add_common_options(DATABRICKS_OPTIONS)
 @add_common_options(DATABRICKS_DATA_COLLECTOR_OPTIONS)
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@@ -389,7 +402,7 @@ def add_databricks_metastore(ctx, name, **kwargs):
 @integrations.command(help='Setup a Databricks Delta integration. For metadata queries on delta tables when using an external metastore in databricks.')
 @click.pass_obj
 @add_common_options(DATABRICKS_OPTIONS)
-@add_common_options(WAREHOUSE_SELECT_OPTIONS)
+@add_common_options(warehouse_select_option())
 @add_common_options(DATABRICKS_DATA_COLLECTOR_OPTIONS)
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
@@ -407,7 +420,7 @@ def add_databricks_delta(ctx, name, **kwargs):
 @integrations.command(help=f'Setup a Redshift integration. {WAREHOUSE_VERBIAGE}.')
 @click.pass_obj
 @click.option('--port', help='HTTP port.', default=5439, type=click.INT, show_default=True)
-@add_common_options(WAREHOUSE_CREATE_OPTIONS)
+@add_common_options(warehouse_create_option())
 @add_common_options(DATABASE_OPTIONS)
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @add_common_options(NETWORK_OPTIONS)
@@ -434,7 +447,7 @@ def add_redshift(ctx, database, name, **kwargs):
 @click.option('--private-key-passphrase', help=f'User\'s private key passphrase if using key pair auth. This argument is only needed when the private key is encrypted. {PASSWORD_VERBIAGE}.',
               required=False, cls=AdvancedOptions, prompt_if_requested=True,
               mutually_exclusive_options=['password'], required_with_options=['private_key'])
-@add_common_options(WAREHOUSE_CREATE_OPTIONS)
+@add_common_options(warehouse_create_option())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_snowflake(ctx, name, **kwargs):
@@ -447,7 +460,7 @@ def add_snowflake(ctx, name, **kwargs):
 @integrations.command(help=f'Setup a BigQuery integration. {WAREHOUSE_VERBIAGE}.')
 @click.pass_obj
 @click.option('--key-file', help='JSON Key file.', type=click.Path(exists=True), required=True)
-@add_common_options(WAREHOUSE_CREATE_OPTIONS)
+@add_common_options(warehouse_create_option())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
 def add_bigquery(ctx, key_file, name, **kwargs):
@@ -493,51 +506,54 @@ def add_self_hosted_credentials(ctx, mechanism, key, role, name, **kwargs):
     )
 
 
-@integrations.command(help='Toggle S3 events for Airflow task logs.')
+@integrations.command(help='Configure S3 events for Airflow task logs.')
 @click.pass_obj
-@add_common_options(EVENT_OPTIONS)
+@add_common_options(warehouse_create_option(default='airflow-logs-s3'))
+@add_common_options(DISAMBIGUATE_DC_OPTIONS)
 @add_common_options(role_options())
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
-def toggle_airflow_log_events(ctx, toggle, role, **kwargs):
+def configure_airflow_log_events(ctx, **kwargs):
     """
-    Toggle s3 events for Airflow task logs
+    Configure s3 events for Airflow task logs
     """
-    EventsOnboardingService(config=ctx['config']).toggle_event_configuration(enable=toggle,
-                                                                             connection_type=AIRFLOW_LOGS_CONNECTION_TYPE,
-                                                                             event_type=AIRFLOW_LOGS_EVENT_TYPE,
-                                                                             assumable_role=role,
-                                                                             **kwargs)
+    EventsOnboardingService(config=ctx['config']).configure_airflow_events(**kwargs)
 
 
-@integrations.command(help=f'Toggle S3 metadata events. {EVENT_VERBIAGE}.')
+@integrations.command(help=f'Configure S3 metadata events. {EVENT_VERBIAGE}.')
 @click.pass_obj
-@add_common_options(EVENT_OPTIONS)
-# Not using CONNECTION_OPTIONS as the value for "required" is different
-@click.option('--connection-type', help='Type of the integration.', cls=AdvancedOptions,
+@click.option('--connection-type',
+              help='Type of the integration.', cls=AdvancedOptions,
               mutually_exclusive_options=['connection_id'],
-              type=click.Choice(
-                  [HIVE_MYSQL_CONNECTION_TYPE, GLUE_CONNECTION_TYPE, DATABRICKS_METASTORE_CONNECTION_TYPE],
-                  case_sensitive=False))
+              type=click.Choice([
+                  DATABRICKS_METASTORE_CONNECTION_TYPE,
+                  GLUE_CONNECTION_TYPE,
+                  HIVE_MYSQL_CONNECTION_TYPE
+              ], case_sensitive=False),
+              required=True)
+@add_common_options(warehouse_create_option(default='s3-metadata-events'))
+@add_common_options(DISAMBIGUATE_DC_OPTIONS)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
-def toggle_metadata_events(ctx, toggle, **kwargs):
+def configure_metadata_events(ctx, **kwargs):
     """
-    Toggle s3 metadata events for a lake
+    Configure S3 metadata events for a lake
     """
-    EventsOnboardingService(config=ctx['config']).toggle_event_configuration(enable=toggle,
-                                                                             event_type=S3_METADATA_EVENT_TYPE,
-                                                                             **kwargs)
+    EventsOnboardingService(config=ctx['config']).configure_metadata_events(**kwargs)
 
 
-@integrations.command(help=f'Toggle S3 query log events. {EVENT_VERBIAGE}.')
+@integrations.command(help=f'Configure S3 query log events. {EVENT_VERBIAGE}.')
 @click.pass_obj
-@add_common_options(EVENT_OPTIONS)
+@click.option('--connection-type',
+              help='Type of the integration.',
+              type=click.Choice([
+                  HIVE_S3_CONNECTION_TYPE,
+                  PRESTO_S3_CONNECTION_TYPE
+              ], case_sensitive=True),
+              required=True)
+@add_common_options(warehouse_select_option())
 @add_common_options(role_options())
-@click.option('--connection-type', help='Type of the integration.',
-              type=click.Choice([HIVE_S3_CONNECTION_TYPE, PRESTO_S3_CONNECTION_TYPE], case_sensitive=True),
-              required=False)
 @click.option('--format-type', help='Query log format.',
               type=click.Choice(['hive-emr', 'hive-native', 'custom'], case_sensitive=True),
-              required=False)
+              required=True)
 @click.option('--source-format', help='Query log file format. Only required when "custom" is used.',
               type=click.Choice(['json', 'jsonl'], case_sensitive=True),
               required=False)
@@ -546,14 +562,44 @@ def toggle_metadata_events(ctx, toggle, **kwargs):
               type=click.Path(exists=True),
               required=False)
 @click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
-def toggle_ql_events(ctx, toggle, role, **kwargs):
+def configure_query_log_events(ctx, **kwargs):
     """
-    Toggle s3 query log events for a lake
+    Configure S3 query log events for a lake
     """
-    EventsOnboardingService(config=ctx['config']).toggle_event_configuration(enable=toggle,
-                                                                             event_type=S3_QL_EVENT_TYPE,
-                                                                             assumable_role=role,
-                                                                             **kwargs)
+    EventsOnboardingService(config=ctx['config']).configure_query_log_events(**kwargs)
+
+
+@integrations.command(help=f'Disable S3 events for Airflow task logs.')
+@click.pass_obj
+@click.option('--name', help='Resource name (only required if more than one exists)')
+@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
+def disable_airflow_log_events(ctx, **kwargs):
+    """
+    Disable S3 events for Airflow task logs
+    """
+    EventsOnboardingService(config=ctx['config']).disable_airflow_log_events(**kwargs)
+
+
+@integrations.command(help=f'Disable S3 metadata events.')
+@click.pass_obj
+@click.option('--name', help='Resource name (only required if more than one exists)')
+@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
+def disable_metadata_events(ctx, **kwargs):
+    """
+    Configure S3 metadata events
+    """
+    EventsOnboardingService(config=ctx['config']).disable_metadata_events(**kwargs)
+
+
+@integrations.command(help=f'Disable S3 query log events.')
+@click.pass_obj
+@click.option('--name', help='Resource name (only required if more than one exists)')
+@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
+def disable_query_log_events(ctx, **kwargs):
+    """
+    Configure S3 query log events
+    """
+    EventsOnboardingService(config=ctx['config']).disable_query_log_events(**kwargs)
 
 
 @integrations.command(help=f'Setup a Tableau integration. {BI_VERBIAGE}.')
@@ -646,7 +692,7 @@ def add_power_bi(ctx, **kwargs):
               required=True, cls=AdvancedOptions, prompt_if_requested=True)
 @click.option('--dbt-cloud-account-id', 'dbt_cloud_account_id', help='dbt Cloud Account ID.', required=True)
 @click.option('--dbt-cloud-base-url', 'dbt_cloud_base_url', help='dbt Cloud Base URL.', required=False)
-@add_common_options(WAREHOUSE_SELECT_OPTIONS)
+@add_common_options(warehouse_select_option())
 @add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
 def add_dbt_cloud(ctx, name, **kwargs):
     """
