@@ -137,9 +137,49 @@ mutation testSelfHostedCredentials($connectionType:String, $selfHostingMechanism
 }
 """
 
-TOGGLE_EVENT_MUTATION = """
-mutation toggleEventConfig($dwId:UUID, $enable:Boolean!, $connectionId:UUID, $connectionType:String, $eventType:DataCollectorEventTypes!, $assumableRole:String, $externalId:String, $formatType:String, $sourceFormat:String, $mapping:JSONString) {
-  toggleEventConfig(dwId:$dwId, enable:$enable, connectionId:$connectionId, connectionType:$connectionType, eventType: $eventType, assumableRole:$assumableRole, externalId:$externalId, formatType:$formatType, sourceFormat:$sourceFormat, mapping:$mapping){
+CONFIGURE_AIRFLOW_LOG_EVENTS_MUTATION = """
+mutation configureAirflowLogEvents($role: String!, $externalId: String, $name: String, $dcId: UUID) {
+  configureAirflowLogEvents(assumableRole: $role, externalId: $externalId, name: $name, dcId: $dcId) {
+    success
+  }
+}
+"""
+
+CONFIGURE_METADATA_EVENTS_MUTATION = """
+mutation configureMetadataEvents($connectionType: String!, $name: String, $dcId: UUID) {
+  configureMetadataEvents(connectionType: $connectionType, name: $name, dcId: $dcId) {
+    success
+  }
+}
+"""
+
+CONFIGURE_QUERY_LOG_EVENTS_MUTATION = """
+mutation configureQueryLogEvents($connectionType: String!, $role: String!, $externalId: String, $formatType: String!, $location: String, $mapping: JSONString, $sourceFormat: String, $name: String) {
+  configureQueryLogEvents(connectionType: $connectionType, assumableRole: $role, externalId: $externalId, formatType: $formatType, location: $location, mapping: $mapping, sourceFormat: $sourceFormat, name: $name) {
+    success
+  }
+}
+"""
+
+DISABLE_AIRFLOW_LOG_EVENTS_MUTATION = """
+mutation disableAirflowLogEvents($name: String) {
+  disableAirflowLogEvents(name: $name) {
+    success
+  }
+}
+"""
+
+DISABLE_METADATA_EVENTS_MUTATION = """
+mutation disableMetadataEvents($name: String) {
+  disableMetadataEvents(name: $name) {
+    success
+  }
+}
+"""
+
+DISABLE_QUERY_LOG_EVENTS_MUTATION = """
+mutation disableQueryLogEvents($name: String) {
+  disableQueryLogEvents(name: $name) {
     success
   }
 }
