@@ -25,7 +25,8 @@ _SAMPLE_COLLECTOR_RESPONSE = {
             'templateLaunchUrl': _SAMPLE_VALID_LAUNCH_URL,
             'stackArn': _SAMPLE_STACK_ARN,
             'active': True,
-            'apiGatewayId': _SAMPLE_GATEWAY_ID
+            'apiGatewayId': _SAMPLE_GATEWAY_ID,
+            'templateVariant': 'dionysus'
         }
     }
 }
@@ -126,6 +127,47 @@ class CollectorManagementTest(TestCase):
                                                                      parameters=None)
         self._aws_wrapper_mock.deploy_gateway.assert_called_once_with(gateway_id=_SAMPLE_GATEWAY_ID)
 
+    def test_upgrade_to_janus(self):
+        self._aws_wrapper_mock.get_stack_parameters.return_value = [
+            {'ParameterKey': 'CreateCrossAccountRole', 'ParameterValue': 'True'},
+            {'ParameterKey': 'CreateEventInfra', 'ParameterValue': 'True'},
+            {'ParameterKey': 'EnableGatewayAccessLogging', 'ParameterValue': 'False'},
+            {'ParameterKey': 'ExistingS3Vpce', 'ParameterValue': 'True'},
+            {'ParameterKey': 'ExistingSubnetIds', 'ParameterValue': 'True'},
+            {'ParameterKey': 'ExistingVpcId', 'ParameterValue': 'True'},
+            {'ParameterKey': 'subnetCIDRs', 'ParameterValue': '2222'},
+            {'ParameterKey': 'vpcCIDR', 'ParameterValue': '1111'},
+            {'ParameterKey': 'workerLambdaConcurrentExecutions', 'ParameterValue': '12'},
+        ]
+        self._aws_wrapper_mock.upgrade_stack.return_value = True
+
+        self._request_wrapper_mock.make_request_v2.return_value = _SAMPLE_MONOLITH_RESPONSE
+        self._service.upgrade_template(update_infra=True, new_params={
+            'EnableRemoteUpdates': 'True',
+            'CreateEventInfra': 'False'
+        })
+
+        self._aws_wrapper_mock.get_stack_parameters.assert_called_once_with(stack_id=_SAMPLE_STACK_ARN)
+        self._aws_wrapper_mock.upgrade_stack.assert_called_once_with(
+            stack_id=_SAMPLE_STACK_ARN,
+            template_link=_SAMPLE_TEMPLATE_URI,
+            parameters=[
+                # updated parameter
+                {'ParameterKey': 'CreateEventInfra', 'ParameterValue': 'False', 'UsePreviousValue': False},
+                # preserved parameters
+                {'ParameterKey': 'ExistingS3Vpce', 'UsePreviousValue': True},
+                {'ParameterKey': 'ExistingSubnetIds', 'UsePreviousValue': True},
+                {'ParameterKey': 'ExistingVpcId', 'UsePreviousValue': True},
+                # migrated parameters
+                {'ParameterKey': 'SubnetCIDRs', 'ParameterValue': '2222'},
+                {'ParameterKey': 'VpcCIDR', 'ParameterValue': '1111'},
+                {'ParameterKey': 'WorkerLambdaConcurrentExecutions', 'ParameterValue': '12'},
+                # new parameter
+                {'ParameterKey': 'EnableRemoteUpdates', 'ParameterValue': 'True'},
+            ]
+        )
+        self._aws_wrapper_mock.deploy_gateway.assert_called_once_with(gateway_id=_SAMPLE_GATEWAY_ID)
+
     def test_failed_stack_upgrade(self):
         self._aws_wrapper_mock.get_stack_parameters.return_value = None
         self._aws_wrapper_mock.upgrade_stack.return_value = False
@@ -229,24 +271,15 @@ class CollectorManagementTest(TestCase):
     @patch('montecarlodata.collector.management.prompt_connection')
     @patch('montecarlodata.collector.management.click')
     def test_add_collector(self, click_mock, prompt_connection_mock, launch_quick_create_link_mock):
-        template_provider = 'provider'
-        template_variant = 'variant'
         mocked_region = 'us-east-1'
         click_mock.prompt.return_value = mocked_region
         self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(data=Box({'dc': {'uuid': '1234'}}))
 
-        self._service.add_collector(
-            template_provider=template_provider,
-            template_variant=template_variant
-        )
+        self._service.add_collector()
 
         self._request_wrapper_mock.make_request_v2.assert_called_once_with(
             query=ADD_COLLECTOR_RECORD,
-            operation='createCollectorRecord',
-            variables = {
-                'template_provider': template_provider,
-                'template_variant': template_variant
-            }
+            operation='createCollectorRecord'
         )
         click_mock.echo.assert_called_once_with("Created collector record with ID '1234'")
         prompt_connection_mock.assert_called_once_with(message=ADD_DC_PROMPT_VERBIAGE)
@@ -258,21 +291,12 @@ class CollectorManagementTest(TestCase):
 
     @patch('montecarlodata.collector.management.click')
     def test_add_collector_no_prompt(self, click_mock):
-        template_provider = 'provider'
-        template_variant = 'variant'
-
         self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(data=Box({'dc': {'uuid': '1234'}}))
 
-        self._service.add_collector(template_provider=template_provider,
-                                    template_variant=template_variant,
-                                    no_prompt=True)
+        self._service.add_collector(no_prompt=True)
 
         self._request_wrapper_mock.make_request_v2.assert_called_once_with(
             query=ADD_COLLECTOR_RECORD,
             operation='createCollectorRecord',
-            variables = {
-                'template_provider': template_provider,
-                'template_variant': template_variant
-            }
         )
         click_mock.echo.assert_called_once_with("Created collector record with ID '1234'")
