@@ -2,12 +2,13 @@ import copy
 import dataclasses
 import json
 from unittest import TestCase
-from unittest.mock import Mock, patch, MagicMock, call, ANY
+from unittest.mock import Mock, patch, MagicMock, call, ANY, PropertyMock
+from io import StringIO 
 
 import click
 from box import Box, BoxList
 
-from montecarlodata.common.data import EventProperties, AWSArn, DcResourceProperties
+from montecarlodata.common.data import EventProperties, ResourceProperties, AWSArn, DcResourceProperties, BucketRow
 from montecarlodata.common.resources import CloudResourceService
 from montecarlodata.common.user import UserService
 from montecarlodata.utils import AwsClientWrapper
@@ -48,6 +49,13 @@ _SAMPLE_EMR_STATUS = {'State': _SAMPLE_EMR_STATE}
 _SAMPLE_EMR_LOG_URI = 's3://bucket/path'
 _SAMPLE_EMR_LOG_URI_2 = 's3://bucket_2/path'
 
+_SAMPLE_BUCKETS_FILE = """bucket_name,prefix,suffix
+foo,,
+bar,prefix1,
+baz,,suffix1
+qux,prefix2,suffix2
+"""
+
 _SAMPLE_EVENT_PAYLOAD = {
     'bucket_name': 'mcd-test-events',
     'collector_aws_profile': 'dev',
@@ -57,6 +65,15 @@ _SAMPLE_EVENT_PAYLOAD = {
     'bucket_aws_profile': None,
     'event_type': 'metadata'
 }
+_SAMPLE_BUCKETS_FILE_EVENT_PAYLOAD = {
+    'buckets_filename': 'buckets.csv',
+    'collector_aws_profile': 'dev',
+    'prefix': None,
+    'suffix': None,
+    'topic_arn': None,
+    'bucket_aws_profile': None,
+    'event_type': 'query-logs'
+}
 _SAMPLE_COLLECTOR = {
     'uuid': 'e34eaf86-092d-47ac-97c3-cfa8b6a6e7fe',
     'stackArn': 'arn:aws:cloudformation:us-east-1:123456789:stack/loki/99',
@@ -69,16 +86,31 @@ _SAMPLE_COLLECTOR = {
 
 _SAMPLE_DC_CLIENT = Mock()
 _SAMPLE_CUST_RESOURCES_CLIENT = Mock()
+
+_SAMPLE_RESOURCE_PROPERTY_1 = ResourceProperties(
+    bucket_name='mcd-test-events',
+    client=copy.deepcopy(_SAMPLE_CUST_RESOURCES_CLIENT),
+    account_id='123456789',
+    region='us-east-1'
+)
+_SAMPLE_RESOURCE_PROPERTY_2 = ResourceProperties(
+    bucket_name='mcd-test-events-us-east-2',
+    client=copy.deepcopy(_SAMPLE_CUST_RESOURCES_CLIENT),
+    account_id='123456789',
+    region='us-east-2'
+)
+
+_SAMPLE_RESOURCE_PROPERTIES = [
+    _SAMPLE_RESOURCE_PROPERTY_1,
+    _SAMPLE_RESOURCE_PROPERTY_2
+]
 _SAMPLE_EVENT_DETAILS = EventProperties(
     event_type='metadata',
-    bucket_name='mcd-test-events',
     event_queue_arn='arn:aws:sqs:us-east-1:123456789:loki-MetadataEventQueue-42',
     collection_region='us-east-1',
-    cust_resources_region='us-east-1',
     collection_account_id='123456789',
-    cust_account_id='123456789',
     collection_client=_SAMPLE_DC_CLIENT,
-    cust_resources_client=_SAMPLE_CUST_RESOURCES_CLIENT
+    resource_properties=_SAMPLE_RESOURCE_PROPERTIES
 )
 
 
@@ -218,10 +250,6 @@ class CloudResourcesTest(TestCase):
         mock_click.assert_has_calls(
             [
                 call.echo('[Gathering collector and resource details]'),
-                call.echo('[Creating or updating topic properties]'),
-                call.echo('[Updating event queue properties]'),
-                call.echo('[Creating topic subscription]'),
-                call.echo('[Creating event notification]'),
                 call.echo('\nProcess complete. Have a nice day!')
             ]
         )
@@ -241,39 +269,121 @@ class CloudResourcesTest(TestCase):
     def test_initialize_events(self, mock_get_q, aws_wrapper):
         derived_bucket_location, dc_outputs, q_arn = 'thor', {'foo': 'bar'}, 'arn:aws:sqs:us-east-1:12345:odin'
         aws_wrapper().get_bucket_location.return_value = derived_bucket_location
+        aws_wrapper().region = derived_bucket_location
         aws_wrapper().get_stack_outputs.return_value = dc_outputs
         aws_wrapper().get_caller_identity.return_value = AWSArn(q_arn).account
 
         mock_get_q.return_value = q_arn
 
-        self.assertEqual(
-            self._service._initialize_events(collector_props=Box(_SAMPLE_COLLECTOR), **_SAMPLE_EVENT_PAYLOAD),
-            EventProperties(
+        answer = self._service._initialize_events(collector_props=Box(_SAMPLE_COLLECTOR), **_SAMPLE_EVENT_PAYLOAD)
+        expected = EventProperties(
                 event_type=_SAMPLE_EVENT_PAYLOAD['event_type'],
-                bucket_name=_SAMPLE_EVENT_PAYLOAD['bucket_name'],
                 event_queue_arn=q_arn,
                 collection_region=AWSArn(q_arn).region,
-                cust_resources_region=derived_bucket_location,
                 collection_account_id=AWSArn(_SAMPLE_COLLECTOR['stackArn']).account,
-                cust_account_id=AWSArn(q_arn).account,
                 collection_client=ANY,
-                cust_resources_client=ANY
+                resource_properties=[
+                    ResourceProperties(
+                        account_id=AWSArn(q_arn).account,
+                        region=derived_bucket_location,
+                        bucket_name=_SAMPLE_EVENT_PAYLOAD['bucket_name'],
+                        client=ANY
+                    )
+                ]
             )
-        )
+        self.assertEqual(answer, expected)
 
         aws_wrapper.assert_has_calls(
             [
-                call(profile_name=_SAMPLE_CONFIG.aws_profile, region_name=_SAMPLE_CONFIG.aws_region),
-                call().get_bucket_location(bucket_name='mcd-test-events'),
                 call(profile_name=_SAMPLE_EVENT_PAYLOAD['collector_aws_profile'],
                      region_name=_SAMPLE_CONFIG.aws_region),
-                call(profile_name=_SAMPLE_CONFIG.aws_profile, region_name=derived_bucket_location),
                 call().get_stack_outputs(stack_id=_SAMPLE_COLLECTOR['stackArn']),
+                call(profile_name=_SAMPLE_CONFIG.aws_profile),
+                call().get_bucket_location(bucket_name='mcd-test-events'),
+                call(profile_name=_SAMPLE_CONFIG.aws_profile, region_name=derived_bucket_location),
                 call().get_caller_identity()
             ]
         )
         mock_get_q.assert_called_once_with('metadata', dc_outputs)
 
+    @patch('montecarlodata.common.resources.AwsClientWrapper')
+    @patch.object(CloudResourceService, '_get_relevant_q')
+    @patch.object(CloudResourceService, 'parse_buckets_file')
+    def test_initialize_events_from_buckets_file(self, mock_parse_buckets, mock_get_q, aws_wrapper):
+        derived_bucket_location, dc_outputs, q_arn = 'thor', {'foo': 'bar'}, 'arn:aws:sqs:us-east-1:12345:odin'
+        aws_wrapper().get_bucket_location.return_value = derived_bucket_location
+        aws_wrapper().region = derived_bucket_location
+        aws_wrapper().get_stack_outputs.return_value = dc_outputs
+        aws_wrapper().get_caller_identity.return_value = AWSArn(q_arn).account
+
+        mock_get_q.return_value = q_arn
+        mock_parse_buckets.return_value = [
+            BucketRow(bucket_name="foo"),
+            BucketRow(bucket_name="bar"),
+            BucketRow(bucket_name="baz")
+        ]
+
+        answer = self._service._initialize_events(collector_props=Box(_SAMPLE_COLLECTOR), **_SAMPLE_BUCKETS_FILE_EVENT_PAYLOAD)
+        expected = EventProperties(
+                event_type=_SAMPLE_BUCKETS_FILE_EVENT_PAYLOAD['event_type'],
+                event_queue_arn=q_arn,
+                collection_region=AWSArn(q_arn).region,
+                collection_account_id=AWSArn(_SAMPLE_COLLECTOR['stackArn']).account,
+                collection_client=ANY,
+                resource_properties=[
+                    ResourceProperties(
+                        account_id=AWSArn(q_arn).account,
+                        region=derived_bucket_location,
+                        bucket_name='foo',
+                        client=ANY
+                    ),
+                    ResourceProperties(
+                        account_id=AWSArn(q_arn).account,
+                        region=derived_bucket_location,
+                        bucket_name='bar',
+                        client=ANY
+                    ),
+                    ResourceProperties(
+                        account_id=AWSArn(q_arn).account,
+                        region=derived_bucket_location,
+                        bucket_name='baz',
+                        client=ANY
+                    )
+                ]
+            )
+
+        self.assertEqual(answer, expected)
+
+        aws_wrapper.assert_has_calls(
+            [
+                call(profile_name=_SAMPLE_EVENT_PAYLOAD['collector_aws_profile'],
+                     region_name=_SAMPLE_CONFIG.aws_region),
+                call().get_stack_outputs(stack_id=_SAMPLE_COLLECTOR['stackArn']),
+                call(profile_name=_SAMPLE_CONFIG.aws_profile),
+                call().get_bucket_location(bucket_name='foo'),
+                call(profile_name=_SAMPLE_CONFIG.aws_profile, region_name=derived_bucket_location),
+                call().get_bucket_location(bucket_name='bar'),
+                call().get_bucket_location(bucket_name='baz'),
+                call().get_caller_identity(),
+                call().get_caller_identity(),
+                call().get_caller_identity()
+            ]
+        )
+        mock_get_q.assert_called_once_with('query-logs', dc_outputs)
+
+    @patch('builtins.open')
+    def test_parse_buckets_file(self, mock_open):
+        mock_open.return_value = StringIO(_SAMPLE_BUCKETS_FILE)
+        answer = self._service.parse_buckets_file("test_filename")
+        expected = [
+            BucketRow(bucket_name='foo', prefix='', suffix=''),
+            BucketRow(bucket_name='bar', prefix='prefix1', suffix=''),
+            BucketRow(bucket_name='baz', prefix='', suffix='suffix1'),
+            BucketRow(bucket_name='qux', prefix='prefix2', suffix='suffix2')
+        ]
+
+        self.assertEqual(answer, expected)
+
     def test_get_relevant_q(self):
         m_out, q_out = 'loki', 'thor'
         sample_outputs = [
@@ -292,66 +402,132 @@ class CloudResourcesTest(TestCase):
     @patch('montecarlodata.common.resources.prompt_connection')
     @patch.object(CloudResourceService, '_extend_topic_policy')
     @patch.object(CloudResourceService, '_is_preexisting_topic_policy')
-    def test_create_or_update_event_topic(self, mock_is_preexisting, mock_extend_policy, mock_prompt):
-        attributes = {'foo': 'bar'}
-        topic_name = 'monte-carlo-data-us-east-1-metadata-events-topic'
-        topic_arn = 'arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic'
-        _SAMPLE_CUST_RESOURCES_CLIENT.create_sns_topic.return_value = topic_arn
-        _SAMPLE_CUST_RESOURCES_CLIENT.get_topic_attributes.return_value = attributes
+    @patch.object(CloudResourceService, '_get_preexisting_topic_policy')
+    @patch.object(CloudResourceService, '_save_event_properties')
+    def test_create_or_update_event_topic(self, mock_save_event_properties, mock_get_preexisting, mock_is_preexisting, mock_extend_policy, mock_prompt):
+        attributes = Box({'Condition': {'StringEquals': {'aws:SourceArn': 'foo'}}})
+
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
+        for resource in events.resource_properties:
+            region = resource.region
+            topic_name = f'monte-carlo-data-{region}-metadata-events-topic'
+            topic_arn = f'arn:aws:sns:{region}:12345:monte-carlo-data-{region}-metadata-events-topic'
+            resource.client.create_sns_topic.return_value = topic_arn
+            resource.client.get_topic_attributes.return_value = attributes
+        
+        mock_get_preexisting.return_value = attributes
         mock_is_preexisting.return_value = False
         mock_extend_policy.return_value = attributes
 
-        self._service._create_or_update_event_topic(event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=True)
+        self._service._create_or_update_event_topic(event_details=events, skip_prompts=True)
+
         mock_is_preexisting.assert_has_calls(
             [
-                call(
-                    topic_policy_id='monte-carlo-data-metadata-mcd-test-events-ID',
-                    topic_attributes={'foo': 'bar'}
-                ),
                 call(
                     topic_policy_id='monte-carlo-data-metadata-loki-MetadataEventQueue-42-ID',
-                    topic_attributes={'foo': 'bar'}
+                    topic_attributes=attributes
+                ),
+                call( 
+                    topic_policy_id='monte-carlo-data-metadata-loki-MetadataEventQueue-42-ID',
+                    topic_attributes=attributes
                 )
             ]
         )
-        mock_extend_policy.assert_called_once_with(
-            topic_attributes={'foo': 'bar'},
-            new_statements=[
-                {'Sid': 'monte-carlo-data-metadata-mcd-test-events-ID', 'Effect': 'Allow', 'Principal': {'AWS': '*'},
-                 'Action': 'SNS:Publish',
-                 'Resource': 'arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic',
-                 'Condition': {'StringEquals': {'aws:SourceArn': 'arn:aws:s3:::mcd-test-events'}}},
-                {'Sid': 'monte-carlo-data-metadata-loki-MetadataEventQueue-42-ID', 'Effect': 'Allow',
-                 'Principal': {'AWS': '123456789'}, 'Action': 'sns:Subscribe',
-                 'Resource': 'arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic'}]
+        mock_extend_policy.assert_has_calls(
+            [
+                call(
+                    topic_attributes=Box({"Condition": {"StringEquals": {"aws:SourceArn": "foo"}}}),
+                    new_statements=[
+                        {
+                            "Sid": "monte-carlo-data-metadata-loki-MetadataEventQueue-42-ID",
+                            "Effect": "Allow",
+                            "Principal": {"AWS": "123456789"},
+                            "Action": "sns:Subscribe",
+                            "Resource": "arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic",
+                        }
+                    ],
+                    update_statements={
+                        "monte-carlo-data-metadata-monte-carlo-data-us-east-1-metadata-events-topic-ID": Box(
+                            {
+                                "Condition": {
+                                    "StringEquals": {
+                                        "aws:SourceArn": ["foo", "arn:aws:s3:::mcd-test-events"]
+                                    }
+                                }
+                            }
+                        )
+                    },
+                ),
+                call(
+                    topic_attributes=Box({"Condition": {"StringEquals": {"aws:SourceArn": "foo"}}}),
+                    new_statements=[
+                        {
+                            "Sid": "monte-carlo-data-metadata-loki-MetadataEventQueue-42-ID",
+                            "Effect": "Allow",
+                            "Principal": {"AWS": "123456789"},
+                            "Action": "sns:Subscribe",
+                            "Resource": "arn:aws:sns:us-east-2:12345:monte-carlo-data-us-east-2-metadata-events-topic",
+                        }
+                    ],
+                    update_statements={
+                        "monte-carlo-data-metadata-monte-carlo-data-us-east-2-metadata-events-topic-ID": Box(
+                            {
+                                "Condition": {
+                                    "StringEquals": {
+                                        "aws:SourceArn": [
+                                            "foo",
+                                            "arn:aws:s3:::mcd-test-events-us-east-2",
+                                        ]
+                                    }
+                                }
+                            }
+                        )
+                    },
+                )            
+            ]
         )
-        _SAMPLE_CUST_RESOURCES_CLIENT.set_topic_attributes.assert_called_once_with(
-            arn=topic_arn,
-            name='Policy',
-            value=attributes
-        )
-        _SAMPLE_CUST_RESOURCES_CLIENT.get_topic_attributes.assert_called_once_with(arn=topic_arn)
-        _SAMPLE_CUST_RESOURCES_CLIENT.create_sns_topic.assert_called_once_with(name=topic_name)
+
+        for resource in events.resource_properties:
+            resource.client.set_topic_attributes.assert_called_once_with(
+                arn=resource.topic_arn,
+                name='Policy',
+                value=attributes
+            )
+            resource.client.get_topic_attributes.assert_called_once_with(arn=resource.topic_arn)
+            resource.client.create_sns_topic.assert_called_once_with(name=resource.topic_name)
         mock_prompt.assert_has_calls(
             [
                 call(
-                    message=f"Create topic '{topic_name}' in 'us-east-1' (123456789)? "
+                    message=f"Create topic 'monte-carlo-data-us-east-1-metadata-events-topic' in 'us-east-1' (123456789)? "
                             "This is an idempotent operation.",
                     skip_prompt=True
                 ),
+                call(message='Please confirm', skip_prompt=True),
+                call(
+                    message="Create topic 'monte-carlo-data-us-east-2-metadata-events-topic' in 'us-east-2' (123456789)? "
+                            "This is an idempotent operation.",
+                    skip_prompt=True,
+                ),                
                 call(message='Please confirm', skip_prompt=True)
             ]
         )
 
-    def test_create_or_update_event_topic_invalid(self):
-        events = dataclasses.replace(_SAMPLE_EVENT_DETAILS)
+        mock_save_event_properties.assert_called()
+
+    def test_create_or_update_event_topic_arn_invalid(self):
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
 
         with self.assertRaises(click.exceptions.Abort):
-            events.topic_arn = 'foo'
+            for resource in events.resource_properties:
+                resource.topic_arn = 'foo'
             self._service._create_or_update_event_topic(event_details=events, skip_prompts=True)  # invalid ARN
 
+    def test_create_or_update_event_topic_region_invalid(self):
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
+
         with self.assertRaises(click.exceptions.Abort):
-            events.topic_arn = 'arn:aws:sns:us-east-2:12345:monte-carlo-data-us-east-1-metadata-events-topic'
+            for resource in events.resource_properties:
+                resource.topic_arn = 'arn:aws:sns:us-west-1:12345:monte-carlo-data-us-east-1-metadata-events-topic'
             self._service._create_or_update_event_topic(event_details=events, skip_prompts=True)  # region mismatch
 
     def test_preexisting_topic_policy(self):
@@ -365,18 +541,32 @@ class CloudResourcesTest(TestCase):
         topic_attributes = Box({'Attributes': {'Policy': '{"Statement": [{"Sid": "foo"}]}'}})
         new_statements = [{'x': 'y'}]
         self.assertEqual(
-            self._service._extend_topic_policy(topic_attributes, new_statements),
+            self._service._extend_topic_policy(topic_attributes, new_statements, update_statements=[]),
             '{"Statement": [{"Sid": "foo"}, {"x": "y"}]}'
         )
 
+    def test_extend_topic_policy_with_updates(self):
+        topic_attributes = Box({'Attributes': {'Policy': '{"Statement": [{"Sid": "foo"}]}'}})
+        new_statements = [{'x': 'y'}]
+        update_statements = {'foo': {'Sid': 'foo', 'bar': 'baz'}}
+        self.assertEqual(
+            self._service._extend_topic_policy(topic_attributes, new_statements, update_statements=update_statements),
+            '{"Statement": [{"Sid": "foo", "bar": "baz"}, {"x": "y"}]}'
+        )
+
     @patch('montecarlodata.common.resources.prompt_connection')
-    def test_update_event_q(self, mock_prompt):
+    @patch.object(CloudResourceService, '_save_event_properties')
+    def test_update_event_q(self, mock_save_event_properties, mock_prompt):
         name = 'loki-MetadataEventQueue-42'
         _SAMPLE_DC_CLIENT.get_q_attributes.return_value = None
 
-        self._service._update_event_q(event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=True)
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
+        for resource in events.resource_properties:
+            resource.topic_arn = f'arn:aws:sns:{resource.region}:12345:monte-carlo-data-{resource.region}-metadata-events-topic'
 
-        _SAMPLE_DC_CLIENT.set_q_attributes.assert_called_once_with(
+        self._service._update_event_q(event_details=events, skip_prompts=True)
+
+        events.collection_client.set_q_attributes.assert_called_once_with(
             name=name,
             attributes={
                 'Policy': '{"Version": "2008-10-17", "Statement": [{"Sid": "__owner", "Effect": "Allow", "Principal": '
@@ -385,15 +575,18 @@ class CloudResourcesTest(TestCase):
                           '{"Sid": "__sender", "Effect": "Allow", "Principal": {"AWS": "*"}, "Action": '
                           '"SQS:SendMessage", "Resource": "arn:aws:sqs:us-east-1:123456789:loki-MetadataEventQueue-42",'
                           ' "Condition": {"ArnLike": {"aws:SourceArn": '
-                          '["arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic"]}}}]}'
+                          '["arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic",'
+                          ' "arn:aws:sns:us-east-2:12345:monte-carlo-data-us-east-2-metadata-events-topic"]}}}]}'
             }
         )
-        _SAMPLE_DC_CLIENT.get_q_attributes.assert_called_once_with(name=name, attributes=['Policy'])
+        events.collection_client.get_q_attributes.assert_called_once_with(name=name, attributes=['Policy'])
         mock_prompt.assert_called_once_with(message='Please confirm', skip_prompt=True)
+        mock_save_event_properties.assert_called()
 
     @patch('montecarlodata.common.resources.prompt_connection')
     @patch.object(CloudResourceService, '_extend_q_policy')
-    def test_update_event_q_with_existing(self, mock_extend, mock_prompt):
+    @patch.object(CloudResourceService, '_save_event_properties')
+    def test_update_event_q_with_existing(self, mock_save_event_properties, mock_extend, mock_prompt):
         name, mock_extend.return_value = 'loki-MetadataEventQueue-42', {'foo': 'bar'}
 
         _SAMPLE_DC_CLIENT.get_q_attributes.return_value = Box({'Attributes': {"Policy": '{}'}})
@@ -402,6 +595,7 @@ class CloudResourcesTest(TestCase):
         _SAMPLE_DC_CLIENT.get_q_attributes.assert_called_once_with(name=name, attributes=['Policy'])
         _SAMPLE_DC_CLIENT.set_q_attributes.assert_called_once_with(name=name, attributes={'Policy': '{"foo": "bar"}'})
         mock_prompt.assert_called_once_with(message='Please confirm', skip_prompt=True)
+        mock_save_event_properties.assert_called()
 
     @patch('montecarlodata.common.resources.prompt_connection')
     @patch.object(CloudResourceService, '_extend_q_policy')
@@ -416,151 +610,197 @@ class CloudResourcesTest(TestCase):
         mock_prompt.assert_not_called()
 
     def test_extend_q_policy(self):
-        events = dataclasses.replace(_SAMPLE_EVENT_DETAILS)
-        events.topic_arn = 'bar'
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
+        for resource in events.resource_properties:
+            resource.topic_arn = 'bar'
 
         # Existing is string
         existing_policy = Box({'Statement': [{'Sid': '__sender', 'Condition': {'ArnLike': {'aws:SourceArn': 'foo'}}}]})
         self.assertEqual(
-            self._service._extend_q_policy(existing_policy=existing_policy, event_details=events),
+            self._service._extend_q_policy(existing_policy=existing_policy, event_details=events, topic_arns=['bar']),
             {'Statement': [{'Sid': '__sender', 'Condition': {'ArnLike': {'aws:SourceArn': ['foo', 'bar']}}}]}
         )
 
         # Existing is list
         existing_policy = Box({'Statement': [{'Sid': '__sender', 'Condition': {'ArnLike': {'aws:SourceArn': ['foo', 'qux']}}}]})
         self.assertEqual(
-            self._service._extend_q_policy(existing_policy=existing_policy, event_details=events),
+            self._service._extend_q_policy(existing_policy=existing_policy, event_details=events, topic_arns=['bar']),
             {'Statement': [{'Sid': '__sender', 'Condition': {'ArnLike': {'aws:SourceArn': ['foo', 'qux', 'bar']}}}]}
         )
 
         # Existing has duplicate
         existing_policy = Box({'Statement': [{'Sid': '__sender', 'Condition': {'ArnLike': {'aws:SourceArn': 'bar'}}}]})
         self.assertEqual(
-            self._service._extend_q_policy(existing_policy=existing_policy, event_details=events),
+            self._service._extend_q_policy(existing_policy=existing_policy, event_details=events, topic_arns=['bar']),
             {'Statement': [{'Sid': '__sender', 'Condition': {'ArnLike': {'aws:SourceArn': 'bar'}}}]}
         )
         existing_policy = Box({'Statement': [{'Sid': '__sender', 'Condition': {'ArnLike': {'aws:SourceArn': ['foo', 'bar']}}}]})
         self.assertEqual(
-            self._service._extend_q_policy(existing_policy=existing_policy, event_details=events),
+            self._service._extend_q_policy(existing_policy=existing_policy, event_details=events, topic_arns=['foo', 'bar']),
             {'Statement': [{'Sid': '__sender', 'Condition': {'ArnLike': {'aws:SourceArn': ['foo', 'bar']}}}]}
         )
 
         # Incompatible format
         with self.assertRaises(click.exceptions.Abort):
-            self._service._extend_q_policy(existing_policy=Box({'Statement': [{'Sid': 'unk'}]}), event_details=events)
+            self._service._extend_q_policy(existing_policy=Box({'Statement': [{'Sid': 'unk'}]}), event_details=events, topic_arns=['bar'])
 
     @patch('montecarlodata.common.resources.prompt_connection')
     @patch.object(CloudResourceService, '_merge_event_notifications')
-    def test_create_event_notification(self, mock_merge_events, mock_prompt):
-        events = dataclasses.replace(_SAMPLE_EVENT_DETAILS)
-        events.topic_arn = 'arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic'
-        events.prefix = 'lake'
-
+    @patch.object(CloudResourceService, '_save_event_properties')
+    def test_create_event_notification(self, mock_save_event_properties, mock_merge_events, mock_prompt):
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
         new_config, existing_config = {'foo': 'bar'}, {'bar': 'foo'}
-        _SAMPLE_CUST_RESOURCES_CLIENT.get_bucket_event_config.return_value = existing_config
+
+        for resource in events.resource_properties:
+            resource.topic_arn = 'arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic'
+            resource.prefix = 'lake'
+            resource.client.get_bucket_event_config.return_value = existing_config
+
         mock_merge_events.return_value = new_config
         self._service._create_event_notification(event_details=events, skip_prompts=True)
 
-        mock_merge_events.assert_called_once_with(
-            setting_id='monte-carlo-data-us-east-1-metadata-events-topic-lake-notification',
-            existing_config=existing_config,
-            event_details=events
-        )
-        _SAMPLE_CUST_RESOURCES_CLIENT.get_bucket_event_config.assert_called_once_with(name=events.bucket_name)
-        _SAMPLE_CUST_RESOURCES_CLIENT.set_bucket_event_config.assert_called_once_with(
-            name=events.bucket_name,
-            notification_config=new_config
-        )
-        mock_prompt.assert_called_once_with(message='Please confirm', skip_prompt=True)
+
+        expected_merge_events_calls = [
+            call(            
+                setting_id='monte-carlo-data-us-east-1-metadata-events-topic-lake-notification',
+                existing_config=existing_config,
+                resource_details=resource
+            ) for resource in events.resource_properties
+        ]
+        self.assertEqual(mock_merge_events.call_args_list, expected_merge_events_calls)
+
+        for resource in events.resource_properties:
+           resource.client.get_bucket_event_config.assert_called_once_with(name=resource.bucket_name)
+           resource.client.set_bucket_event_config.assert_called_once_with(name=resource.bucket_name, notification_config=new_config)
+
+        expected_prompt_calls = [
+            call(            
+                message='Please confirm', skip_prompt=True
+            ) for _ in events.resource_properties
+        ]
+        self.assertEqual(mock_prompt.call_args_list, expected_prompt_calls)
+
+        mock_save_event_properties.assert_called()
 
     @patch('montecarlodata.common.resources.prompt_connection')
     @patch.object(CloudResourceService, '_merge_event_notifications')
-    def test_create_event_notification_with_no_delta(self, mock_merge_events, mock_prompt):
-        events = dataclasses.replace(_SAMPLE_EVENT_DETAILS)
-        events.topic_arn = 'arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic'
-
+    @patch.object(CloudResourceService, '_save_event_properties')
+    def test_create_event_notification_with_no_delta(self, mock_save_event_properties, mock_merge_events, mock_prompt):
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
         new_config, existing_config = {'foo': 'bar'}, {'foo': 'bar'}
-        _SAMPLE_CUST_RESOURCES_CLIENT.get_bucket_event_config.return_value = existing_config
+
+        for resource in events.resource_properties:
+            resource.topic_arn = 'arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic'
+            resource.client.get_bucket_event_config.return_value = existing_config
+
         mock_merge_events.return_value = new_config
         self._service._create_event_notification(event_details=events, skip_prompts=True)
+        expected_merge_events_calls = [
+            call(            
+                setting_id='monte-carlo-data-us-east-1-metadata-events-topic-notification',
+                existing_config=existing_config,
+                resource_details=resource
+            ) for resource in events.resource_properties
+        ]
+        self.assertEqual(mock_merge_events.call_args_list, expected_merge_events_calls)
+
+        for resource in events.resource_properties:
+            resource.client.get_bucket_event_config.assert_called_once_with(name=resource.bucket_name)
+            resource.client.set_bucket_event_config.assert_not_called()
 
-        mock_merge_events.assert_called_once_with(
-            setting_id='monte-carlo-data-us-east-1-metadata-events-topic-notification',
-            existing_config=existing_config,
-            event_details=events
-        )
-        _SAMPLE_CUST_RESOURCES_CLIENT.get_bucket_event_config.assert_called_once_with(name=events.bucket_name)
-        _SAMPLE_CUST_RESOURCES_CLIENT.set_bucket_event_config.assert_not_called()
         mock_prompt.assert_not_called()
+        mock_save_event_properties.assert_called()
 
     def test_merge_event_notifications(self):
         setting_id = 'foo'
-        expected_config = {
-            'Id': setting_id,
-            'TopicArn': 'arn:aws:sns:us-east-1:12345:monte-carlo-data-us-east-1-metadata-events-topic',
-            'Events': ['s3:ObjectCreated:*', 's3:ObjectRemoved:*'],
-            'Filter': {'Key': {'FilterRules': [{'Name': 'Prefix', 'Value': ''}, {'Name': 'Suffix', 'Value': ''}]}}
-        }
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
 
-        # ID already exists
-        existing_setting = {'TopicConfigurations': [{'Id': setting_id}]}
-        self.assertEqual(
-            self._service._merge_event_notifications(setting_id, existing_setting, _SAMPLE_EVENT_DETAILS),
-            existing_setting
-        )
 
-        # Topic already exits
-        existing_setting = {'TopicConfigurations': [{'Id': 'bar'}], 'QueueConfigurations': [{'foo': 'bar'}]}
-        self.assertEqual(
-            self._service._merge_event_notifications(setting_id, existing_setting, _SAMPLE_EVENT_DETAILS),
-            {'TopicConfigurations': [{'Id': 'bar'}, expected_config], 'QueueConfigurations': [{'foo': 'bar'}]}
-        )
+        for resource in events.resource_properties:
+            resource.topic_arn = f'arn:aws:sns:{resource.region}:12345:monte-carlo-data-{resource.region}-metadata-events-topic'
+            expected_config = {
+                'Id': setting_id,
+                'TopicArn': resource.topic_arn,
+                'Events': ['s3:ObjectCreated:*', 's3:ObjectRemoved:*'],
+                'Filter': {'Key': {'FilterRules': [{'Name': 'Prefix', 'Value': ''}, {'Name': 'Suffix', 'Value': ''}]}}
+            }
+            # ID already exists
+            existing_setting = {'TopicConfigurations': [{'Id': setting_id}]}
+            self.assertEqual(
+                self._service._merge_event_notifications(setting_id, existing_setting, resource),
+                existing_setting
+            )
 
-        # No topic config
-        self.assertEqual(
-            self._service._merge_event_notifications(setting_id, {}, _SAMPLE_EVENT_DETAILS),
-            {'TopicConfigurations': [expected_config]}
-        )
-        existing_setting = {'QueueConfigurations': [{'foo': 'bar'}]}
-        self.assertEqual(
-            self._service._merge_event_notifications(setting_id, existing_setting, _SAMPLE_EVENT_DETAILS),
-            {'TopicConfigurations': [expected_config], 'QueueConfigurations': [{'foo': 'bar'}]}
-        )
+            # Topic already exits
+            existing_setting = {'TopicConfigurations': [{'Id': 'bar'}], 'QueueConfigurations': [{'foo': 'bar'}]}
+            self.assertEqual(
+                self._service._merge_event_notifications(setting_id, existing_setting, resource),
+                {'TopicConfigurations': [{'Id': 'bar'}, expected_config], 'QueueConfigurations': [{'foo': 'bar'}]}
+            )
+
+            # No topic config
+            self.assertEqual(
+                self._service._merge_event_notifications(setting_id, {}, resource),
+                {'TopicConfigurations': [expected_config]}
+            )
+
+            existing_setting = {'QueueConfigurations': [{'foo': 'bar'}]}
+            self.assertEqual(
+                self._service._merge_event_notifications(setting_id, existing_setting, resource),
+                {'TopicConfigurations': [expected_config], 'QueueConfigurations': [{'foo': 'bar'}]}
+            )
 
     @patch('montecarlodata.common.resources.prompt_connection')
-    def test_create_topic_subscription(self, mock_prompt):
-        _SAMPLE_CUST_RESOURCES_CLIENT.list_topic_subscriptions.return_value = None
-        self._service._create_topic_subscription(event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=True)
-        _SAMPLE_CUST_RESOURCES_CLIENT.list_topic_subscriptions.assert_called_once_with(
-            arn=_SAMPLE_EVENT_DETAILS.topic_arn
-        )
-        _SAMPLE_CUST_RESOURCES_CLIENT.subscribe_to_topic.assert_called_once_with(
-            arn=_SAMPLE_EVENT_DETAILS.topic_arn,
-            endpoint=_SAMPLE_EVENT_DETAILS.event_queue_arn,
-            attributes={'RawMessageDelivery': 'true'}
-        )
-        mock_prompt.assert_called_once_with(message='Please confirm', skip_prompt=True)
+    @patch.object(CloudResourceService, '_save_event_properties')
+    def test_create_topic_subscription(self, mock_save_event_properties, mock_prompt):
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
+        for resource in events.resource_properties:
+            resource.client.list_topic_subscriptions.return_value = None
+        self._service._create_topic_subscription(event_details=events, skip_prompts=True)
+
+        for resource in events.resource_properties:
+            resource.client.list_topic_subscriptions.assert_called_once_with(
+                arn=resource.topic_arn
+            )
+            resource.client.subscribe_to_topic.assert_called_once_with(
+                arn=resource.topic_arn,
+                endpoint=events.event_queue_arn,
+                attributes={'RawMessageDelivery': 'true'}
+            )
+
+        mock_prompt.assert_has_calls([
+            call(message='Please confirm', skip_prompt=True),
+            call(message='Please confirm', skip_prompt=True),
+        ])
+        mock_save_event_properties.assert_called()
 
     @patch('montecarlodata.common.resources.prompt_connection')
-    def test_create_topic_subscription_with_preexisting_endpoint(self, mock_prompt):
-        _SAMPLE_CUST_RESOURCES_CLIENT.list_topic_subscriptions.return_value = [
-            [
-                {
-                    'SubscriptionArn': 'odin',
-                    'Owner': '12345',
-                    'Protocol': 'sqs',
-                    'Endpoint': _SAMPLE_EVENT_DETAILS.event_queue_arn,
-                    'TopicArn': _SAMPLE_EVENT_DETAILS.topic_arn
-                }
+    @patch.object(CloudResourceService, '_save_event_properties')
+    def test_create_topic_subscription_with_preexisting_endpoint(self, mock_save_event_properties, mock_prompt):
+        events = copy.deepcopy(_SAMPLE_EVENT_DETAILS)
+        for resource in events.resource_properties:
+            resource.client.list_topic_subscriptions.return_value = [
+                [
+                    {
+                        'SubscriptionArn': 'odin',
+                        'Owner': '12345',
+                        'Protocol': 'sqs',
+                        'Endpoint': events.event_queue_arn,
+                        'TopicArn': resource.topic_arn
+                    }
+                ]
             ]
-        ]
 
-        self._service._create_topic_subscription(event_details=_SAMPLE_EVENT_DETAILS, skip_prompts=True)
-        _SAMPLE_CUST_RESOURCES_CLIENT.list_topic_subscriptions.assert_called_once_with(
-            arn=_SAMPLE_EVENT_DETAILS.topic_arn
-        )
-        _SAMPLE_CUST_RESOURCES_CLIENT.subscribe_to_topi.cassert_not_called()
+        self._service._create_topic_subscription(event_details=events, skip_prompts=True)
+
+        for resource in events.resource_properties:
+            resource.client.list_topic_subscriptions.assert_called_once_with(
+                arn=resource.topic_arn
+            )
+            _SAMPLE_CUST_RESOURCES_CLIENT.subscribe_to_topic.cassert_not_called()
+
         mock_prompt.assert_not_called()
+        mock_save_event_properties.assert_called()
 
     @patch('montecarlodata.common.resources.AwsClientWrapper')
     def test_get_dc_resource_props(self, aws_wrapper_mock):
